@@ -17,12 +17,29 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 
 import click
 
-from nucypher.cli.commands import alice, bob, dao, enrico, felix, multisig, stake, status, ursula, worklock, cloudworkers
-from nucypher.cli.painting.help import echo_version
+from nucypher.cli.commands import (
+    alice,
+    bob,
+    dao,
+    enrico,
+    felix,
+    multisig,
+    stake,
+    status,
+    ursula,
+    worklock,
+    cloudworkers
+)
+from nucypher.cli.painting.help import echo_version, echo_config_root_path, echo_logging_root_path
 
 
 @click.group()
-@click.option('--version', help="Echo the CLI version", is_flag=True, callback=echo_version, expose_value=False, is_eager=True)
+@click.option('--version', help="Echo the CLI version",
+              is_flag=True, callback=echo_version, expose_value=False, is_eager=True)
+@click.option('--config-path', help="Echo the configuration root directory path",
+              is_flag=True, callback=echo_config_root_path, expose_value=False, is_eager=True)
+@click.option('--logging-path', help="Echo the logging root directory path",
+              is_flag=True, callback=echo_logging_root_path, expose_value=False, is_eager=True)
 def nucypher_cli():
     """Top level command for all things nucypher."""
 
@@ -65,8 +82,8 @@ ENTRY_POINTS = (
     status.status,      # Network Status
     felix.felix,        # Faucet
     multisig.multisig,  # MultiSig operations
-    worklock.worklock,   # WorkLock
-    cloudworkers.cloudworkers #Remote Worker node management
+    worklock.worklock,         # WorkLock
+    cloudworkers.cloudworkers  # Remote Worker node management
 )
 
 for entry_point in ENTRY_POINTS:
