@@ -31,7 +31,6 @@ from eth_utils import to_checksum_address
 from requests.exceptions import SSLError
 from twisted.internet import reactor, task
 from twisted.internet.defer import Deferred
-from twisted.logger import Logger
 
 import nucypher
 from bytestring_splitter import BytestringSplitter, BytestringSplittingError, PartiallyKwargifiedBytes, \
@@ -102,6 +101,11 @@ class NodeSprout(PartiallyKwargifiedBytes):
     def stamp(self) -> bytes:
         return self.processed_objects['verifying_key'][0]
 
+    @property
+    def domain(self) -> str:
+        domain_bytes = PartiallyKwargifiedBytes.__getattr__(self, "domain")
+        return domain_bytes.decode("utf-8")
+
     @property
     def checksum_address(self):
         if not self._checksum_address:
@@ -210,7 +214,7 @@ class Learner:
         self.log = Logger("learning-loop")  # type: Logger
 
         self.learning_deferred = Deferred()
-        self.learning_domain = domain
+        self.domain = domain
         if not self.federated_only:
             default_middleware = self.__DEFAULT_MIDDLEWARE_CLASS(registry=self.registry)
         else:
@@ -224,7 +228,7 @@ class Learner:
         self._learning_listeners = defaultdict(list)
         self._node_ids_to_learn_about_immediately = set()
 
-        self.__known_nodes = self.tracker_class()
+        self.__known_nodes = self.tracker_class(domain=domain)
         self._verify_node_bonding = verify_node_bonding
 
         self.lonely = lonely
@@ -293,8 +297,8 @@ class Learner:
 
         discovered = []
 
-        if self.learning_domain:
-            canonical_sage_uris = self.network_middleware.TEACHER_NODES.get(self.learning_domain, ())
+        if self.domain:
+            canonical_sage_uris = self.network_middleware.TEACHER_NODES.get(self.domain, ())
 
             for uri in canonical_sage_uris:
                 try:
@@ -350,15 +354,14 @@ class Learner:
         restored_from_disk = []
         invalid_nodes = defaultdict(list)
         for node in stored_nodes:
-            node_domain = node.domain.decode('utf-8')
-            if node_domain != self.learning_domain:
-                invalid_nodes[node_domain].append(node)
+            if node.domain != self.domain:
+                invalid_nodes[node.domain].append(node)
                 continue
             restored_node = self.remember_node(node, record_fleet_state=False)  # TODO: Validity status 1866
             restored_from_disk.append(restored_node)
 
         if invalid_nodes:
-            self.log.warn(f"We're learning about domain '{self.learning_domain}', but found nodes from other domains; "
+            self.log.warn(f"We're learning about domain '{self.domain}', but found nodes from other domains; "
                           f"let's ignore them. These domains and nodes are: {dict(invalid_nodes)}")
 
         return restored_from_disk
@@ -383,7 +386,7 @@ class Learner:
         with suppress(KeyError):
             already_known_node = self.known_nodes[node.checksum_address]
             if not node.timestamp > already_known_node.timestamp:
-                self.log.debug("Skipping already known node {}".format(already_known_node))
+                # self.log.debug("Skipping already known node {}".format(already_known_node))  # FIXME: ""OMG, enough with the learning already!" – @vepkenez  (#1712)
                 # This node is already known.  We can safely return.
                 return False
 
@@ -520,7 +523,7 @@ class Learner:
         except IndexError:
             error = "Not enough nodes to select a good teacher, Check your network connection then node configuration"
             raise self.NotEnoughTeachers(error)
-        self.log.info("Cycled teachers; New teacher is {}".format(self._current_teacher_node))
+        self.log.debug("Cycled teachers; New teacher is {}".format(self._current_teacher_node))
 
     def current_teacher_node(self, cycle=False):
         if cycle:
@@ -556,9 +559,9 @@ class Learner:
         self._learning_deferred = Deferred(canceller=self._discovery_canceller)  # TODO: No longer relevant.
 
         def _discover_or_abort(_first_result):
-            self.log.debug(f"{self} learning at {datetime.datetime.now()}")
+            # self.log.debug(f"{self} learning at {datetime.datetime.now()}")   # 1712
             result = self.learn_from_teacher_node(eager=False, canceller=self._discovery_canceller)
-            self.log.debug(f"{self} finished learning at {datetime.datetime.now()}")
+            # self.log.debug(f"{self} finished learning at {datetime.datetime.now()}")  # 1712
             return result
 
         self._learning_deferred.addCallback(_discover_or_abort)
@@ -781,7 +784,7 @@ class Learner:
         # These except clauses apply to the current_teacher itself, not the learned-about nodes.
         except NodeSeemsToBeDown as e:
             unresponsive_nodes.add(current_teacher)
-            self.log.info(f"Teacher {str(current_teacher)} is perhaps down: {bytes(current_teacher)}:{e}.")
+            self.log.info(f"Teacher {str(current_teacher)} is perhaps down:{e}.")  # FIXME: This was printing the node bytestring. Is this really necessary?  #1712
             return
         except current_teacher.InvalidNode as e:
             # Ugh.  The teacher is invalid.  Rough.
@@ -818,9 +821,9 @@ class Learner:
             self.log.info("Bad response from teacher {}: {} - {}".format(current_teacher, response, response.content))
             return
 
-        if self.learning_domain != current_teacher.serving_domain:
-            self.log.debug(f"{current_teacher} is serving '{current_teacher.serving_domain}', "
-                           f"ignore since we are learning about '{self.learning_domain}'")
+        if self.domain != current_teacher.domain:
+            self.log.debug(f"{current_teacher} is serving '{current_teacher.domain}', "
+                           f"ignore since we are learning about '{self.domain}'")
             return  # This node is not serving our domain.
 
         #
@@ -852,7 +855,7 @@ class Learner:
             current_teacher.update_snapshot(checksum=checksum,
                                             updated=maya.MayaDT(
                                                 int.from_bytes(fleet_state_updated_bytes, byteorder="big")),
-                                            number_of_known_nodes=len(self.known_nodes))
+                                            number_of_known_nodes=self.known_nodes.population())
             return FLEET_STATES_MATCH
 
         # Note: There was previously a version check here, but that required iterating through node bytestrings twice,
@@ -943,7 +946,7 @@ class Teacher:
         # Fleet
         #
 
-        self.serving_domain = domain
+        self.domain = domain
         self.fleet_state_checksum = None
         self.fleet_state_updated = None
         self.last_seen = NEVER_SEEN("No Connection to Node")
@@ -1359,8 +1362,9 @@ class Teacher:
                    "last_seen": last_seen,
                    "fleet_state": node.fleet_state_checksum or 'unknown',
                    "fleet_state_icon": fleet_icon,
-                   "domain": node.learning_domain,
-                   'version': nucypher.__version__}
+                   "domain": node.domain,
+                   'version': nucypher.__version__
+                   }
         return payload
 
     def abridged_node_details(self) -> dict:
